/*
 * This is a demonstration JDBC applet connecting to a selected ODBC
 * data source  via the Easysoft JDBC-ODBC Bridge and displaying the
 * result of a query.
 *
 * The method doTask() can be overwritten in subclasses to cary out
 * other JDBC tasks given an open connection to the selected database.
 *
 * A APPLET tag shown below can be used to launch the applet
 *  
 * <APPLET CODE = "OutputApplet" ARCHIVE ="../jars/EJOB.jar,../jars/qdemo.jar">
 *         <PARAM NAME = "jdbcUrl" VALUE  = "jdbc:easysoft:" />
 *         <PARAM NAME = "query"   VALUE  = "tables" />
 * </APPLET>
 * 
 */

import javax.swing.JApplet;
import javax.swing.JButton;

import java.sql.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class QueryApplet extends JApplet implements Runnable 
{
    private Thread 			worker;
    private JTextArea 		textA=null;
    private JTextArea 		textR=null;
    private String 			message = "Initializing";
    private JTextArea		displayText;
    private JLabel	  		urlLabel;
    private JTextField		urlText;
    private JLabel	  		userLabel;
    private JTextField		userText;
    private JLabel	  		pwdLabel;
    private JPasswordField	pwdText;
	private JButton			connectButton;
	private JFrame			frame;
	private String 			jdbcUrl = "";
	private JScrollPane		scrollableTextA;
	private JScrollPane		scrollableTextR;

    /* 
     * JDBC url to use to connect to. This can be overridden in the 
     * applet parameter: jdbcUrl 
     */

    public  String url = "jdbc:easysoft:";

    /* 
     * query to run: 
     * JOB runs DatabaseMetaData.getTables() when query="tables" 
     * This can be overridden in the applet parameter: query 
     */

    public  String query = "tables";

    public synchronized void start() 
	{
		// Every time "start" is called we create a worker thread to
		// re-evaluate the database query.
		if (worker == null) 
		{	
			message = "Connecting to database";
			worker = new Thread(this);
			worker.start();
		}
    }

    public void  init () 
	{
		// Get query parameter
		String queryParameter = "";
		try 
		{
	    	queryParameter = getParameter("query");
		}
		catch (Exception e) 
	    { // quietly ignore this if not running in browser
	    }

		if (queryParameter!=null && !queryParameter.equals(""))
		{
	    	query=queryParameter;
		}

		// Getting the parameter for jdbcUrl from the calling html file
		// This enables retargeting of the applet 
	
		try 
		{
	    	jdbcUrl = getParameter("jdbcUrl");
		}
		catch (Exception e) 
	    { // quetly ignore this if not running in browser
	    }
	
		if (jdbcUrl!=null && !jdbcUrl.equals("")) 
		{
			url = jdbcUrl;
		}

		JPanel app = new JPanel();					// holds the prompt labels
		app.setLayout(new BorderLayout(5,5));				// overall layout is a NSEWC one

		if (textA==null) 
		{
	    	textA = new JTextArea(message);
			textA.setRows( 2 );
			textA.setEditable( false );
			scrollableTextA = new JScrollPane( textA );
			app.add ( scrollableTextA, BorderLayout.NORTH );
		}  

		if (textR==null) 
		{
			textR = new JTextArea("");
			textR.setEditable( false );
			scrollableTextR = new JScrollPane( textR );
			app.add ( scrollableTextR, BorderLayout.CENTER );
		}
		getContentPane().add(app);
    }

    /*
     * The "run" method is called from the worker thread.  Notice that
     * because this method is doing potentially slow databases accesses
     * we avoid making it a synchronized method.
     */

    public void run() 
	{
		textA.setText( "Getting logon information" );

		try 
		{ 
			displayText = new JTextArea(6,40);

			frame = new JFrame( "Result Set" );
			frame.setSize( 450, 125 );

			// create the components for the applet
			urlLabel = new JLabel ("URL:");

			urlText = new JTextField(jdbcUrl);
			urlText.setFont(new java.awt.Font("dialog", 1, 12));
			urlText.setEditable( false );

			userLabel = new JLabel("Logon User:");

			userText = new JTextField( "" );
			userText.setFont(new java.awt.Font("dialog", 1, 12));

			pwdLabel = new JLabel("Login PWD:");

			pwdText = new JPasswordField( "" );
			pwdText.setFont(new java.awt.Font("dialog", 1, 12));

			connectButton = new JButton("Connect");

			connectButton.addActionListener(new ActionListener() {
					            public void actionPerformed(ActionEvent e)
								{ 
									buttonConnect();
								}});

			// overall layout is 3 panels - top, center and bottom

			JPanel center = new JPanel();					// center part of applet

			getContentPane().setLayout(new BorderLayout(5,5));				// overall layout is a NSEWC one

			// set the top panel - consists of three more horizontally arranged panels

			center.setLayout(new BorderLayout(5, 5));		// give it distinct center

			JPanel topLeft = new JPanel();					// holds the prompt labels
			JPanel topCenter = new JPanel();				// holds the text fields
			JPanel topRight = new JPanel();					// holds the buttons

			center.add ( topLeft, BorderLayout.WEST );
			center.add ( topCenter, BorderLayout.CENTER );
			center.add ( topRight, BorderLayout.EAST );

			// Each of the top panels has a 3 rows x 1 col grid (2 for right panel)

			topLeft.setLayout(new GridLayout (3,1,5,5));
			topCenter.setLayout(new GridLayout (3,1,5,5));
			topRight.setLayout(new GridLayout (3,1,5,5));

			// finally - add all the top components
			topLeft.add(urlLabel);
			topLeft.add(userLabel);
			topLeft.add(pwdLabel);

			topCenter.add(urlText);
			topCenter.add(userText);
			topCenter.add(pwdText);

			topRight.add(connectButton);

			frame.getContentPane().add( center );
			frame.setVisible( true );
		} 
		catch (Exception ex) 
		{
			ex.printStackTrace();
		}
    }
    
    public Connection getJOBConnection() throws SQLException 
	{
		// Load the driver

		try 
		{
			Class.forName("easysoft.sql.jobDriver").newInstance();
		} 
		catch(Exception ex) 
		{
			throw new SQLException("Can't find Database driver class: " + ex);
		}

		// Now establish connection with via the Driver manager
		java.util.Properties p = new java.util.Properties();

		String u = userText.getText();
		if ( u != null )
		{
			p.put( "logonuser", u );
		}

		String s = new String( pwdText.getPassword());
		if ( s != null )
		{
			p.put( "logonpassword",  s );
		}

		textA.setText( "Connecting" );

		Connection con = DriverManager.getConnection(url, p);
		return con;
    }

    /*
     * In subclasses this method can be overwritten to execute different
     * db tasks then this given an open connection to the JDBC data source
     * specified in the applet tag
     */

    public void doTask(Connection con)  throws SQLException 
	{
		// Create a statement Object
		Statement stmt = con.createStatement();
		// Execute a query returning a result set
		ResultSet rs = stmt.executeQuery(query);
		// Get the metadata for the result set
		ResultSetMetaData rsmd = rs.getMetaData();
		// Find out the number of columns in the resultset
		int columnCount = rsmd.getColumnCount();
		//StringBuffer data = new StringBuffer();
		String data = "";
		String rowData ="";
		// While there are rows in the result set
		while (rs.next()) 
		{
			rowData = "";
			// get the data for each column in a row and
			// construct a string of comma separated column values
			for (int i = 1; i<= columnCount; i++) 
			{
				rowData = rowData + rs.getString(i) + ", ";
			}
			data = data + rowData + "\n";
		}

		// Update result set text area
		textR.setText(data);
		// Close result set to release database resources
		rs.close();
		// Close Statement to release database resources
		stmt.close();
		//Close Connection to release database resources
		con.close();
		textA.setText("Run complete");
    }

    /*
     * This private method is used to record an error message for
     * later display.
     */

    synchronized void setError(String mess) 
	{
		textA.setText(mess);
		worker = null;
		// And ask AWT to repaint this applet.
		repaint();
    }

	private void buttonConnect()
	{
		frame.setVisible( false );
		try 
		{
	    	Connection con = getJOBConnection();
	    	doTask(con);	
		} 
		catch(Exception ex) 
		{
	    	setError( ex.getMessage());
		}
	}
}
